/***************************************************************************
                          bibstyledialog.cpp  -  description
                             -------------------
    begin                : Mon Dez 29 2003
    copyright            : (C) 2003 by Nicola Dbelin
    email                : adoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bibstyledialog.h"

#include <qapplication.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qstringlist.h>

BibStyleDialog::BibStyleDialog(QWidget *parent, const char *name ) : BibStyleDlg(parent,name) {

  connect(this, SIGNAL(styleFound(QString)), this, SLOT(insertStyle(QString)));
  scanning = false;
  up = true;
  
}
BibStyleDialog::~BibStyleDialog(){
}

void BibStyleDialog::slotAdd()
{
  if (!lineEdit->text().isEmpty()) {
    listBox->insertItem(lineEdit->text(), -1);
    lineEdit->clear();
    }
}

void BibStyleDialog::slotRemove()
{
  if (QMessageBox::information(this, tr("Clear items"), tr("Do you want to clear\nthe selected items?"),
                               tr("&Yes"), tr("&No"))) return;
  for (int i = (int)listBox->count()-1; i >= 0; i--)
    if (listBox->isSelected(i))
      listBox->removeItem(i);
}

void BibStyleDialog::slotSort()
{
  if (up) {
    listBox->sort(true);
    up = false;
    }
  else {
    listBox->sort(false);
    up = true;
    }
}

void BibStyleDialog::setDirs(QString t, QString l)
{
  tdir = t;
  ltdir = l;
  qDebug("Directory set to: " + t);
  qDebug("Local directory set to: " + l);
}

void BibStyleDialog::slotScanDirs()
{
  qDebug("slotScan");
  if (scanning) scanning = false;

  else {
    listBox->clear();
    scanning = true;
    buttonOk->setEnabled(false);
    buttonCancel->setEnabled(false);
    pushButtonScan->setText(tr("Abort"));

    textLabelStatus->setText(tr("Scanning..."));
    qDebug("Entering search function");
    searchFiles(tdir);
    searchFiles(ltdir);
    textLabelStatus->setText("");
    textLabelDir->setText("");
    scanning = false;
    }

  buttonOk->setEnabled(true);
  buttonCancel->setEnabled(true);
  pushButtonScan->setText(tr("Scan"));

}

void BibStyleDialog::searchFiles(QString dir)
{
  QDir d(dir);
  QStringList t, s;
  QFileInfo f;

//  qDebug("Directory: " + dir);
  textLabelDir->setText(dir);
  d.setFilter(QDir::Dirs | QDir::Files | QDir::NoSymLinks);
  d.setSorting(QDir::Name);

  t = d.entryList();

  for (QStringList::Iterator it = t.begin(); it != t.end(); ++it) {
    if (!scanning) return;
    if ((*it == "..") || (*it == ".")) continue;
    f.setFile(d.absPath() + "/" + *it);
//    qDebug("Scanned file: " + f.absFilePath() + "/" + f.fileName());
    if (f.isFile())
      if (f.extension(false) == "bst") {
//        qDebug("Style found: " + f.absFilePath());
        emit styleFound(f.baseName(true));
        qApp->processEvents(100);
        }
    if (f.isDir())
      searchFiles(f.absFilePath());
    }
}

void BibStyleDialog::insertStyle(QString s)
{
  listBox->insertItem(s, -1);
}
