/***************************************************************************
                          headingsdialog.cpp  -  description
                             -------------------
    begin                : Sam Nov 29 2003
    copyright            : (C) 2003 by Nicola Dbelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "headingsdialog.h"
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qwidgetstack.h>

HeadingsDialog::HeadingsDialog(QWidget *parent, const char *name ) : HeadingsDlg(parent,name) {

  twoside = false;
}

HeadingsDialog::~HeadingsDialog(){
}


void HeadingsDialog::slotReset()
{
  if (QMessageBox::question(this, tr("Restore"),
                                  tr("Do you really want to restore the initial setup?"),
                                  tr("&Yes"), tr("&No") )) return;
  const QStringList lst = listSettings;
  setOptions(lst);
}

void HeadingsDialog::slotClear()
{
  if (QMessageBox::question(this, tr("Clear"),
                                  tr("Do you really want to clear all entries?"),
                                  tr("&Yes"), tr("&No") )) return;
  comboBoxLeh->setCurrentItem(0);
  comboBoxCeh->setCurrentItem(0);
  comboBoxReh->setCurrentItem(0);
  comboBoxLoh->setCurrentItem(0);
  comboBoxCoh->setCurrentItem(0);
  comboBoxRoh->setCurrentItem(0);
  comboBoxLef->setCurrentItem(0);
  comboBoxCef->setCurrentItem(0);
  comboBoxRef->setCurrentItem(0);
  comboBoxLof->setCurrentItem(0);
  comboBoxCof->setCurrentItem(0);
  comboBoxRof->setCurrentItem(0);

  comboBoxIh->setCurrentItem(0);
  comboBoxCh->setCurrentItem(0);
  comboBoxOh->setCurrentItem(0);
  comboBoxIf->setCurrentItem(0);
  comboBoxCf->setCurrentItem(0);
  comboBoxOf->setCurrentItem(0);

  checkBoxHsl->setChecked(false);
  checkBoxFsl->setChecked(false);
  checkBoxHslD->setChecked(false);
  checkBoxFslD->setChecked(false);
}

void HeadingsDialog::setTwoside(bool b) 
{
  twoside = b;
  if (twoside) {
    widgetStack->raiseWidget(1);
    setCaption("Scrpage2 Headings (twoside)");
  }
  else {
    widgetStack->raiseWidget(0);
    setCaption("Scrpage2 Headings (oneside)");
  }
}


// parse a string list containing all current items for combo
// boxes check status of check boxes.
// format of the string list:
// Pos 0: status of headsepline (0 = false, 1 = true)
// Pos 1: status of footsepline (0 = false, 1 = true)
// Pos 2--19: current items of combo boxes
void HeadingsDialog::setOptions(const QStringList &lst)
{
  listSettings = lst;
  int d = 0;
  for (int i = 0; i < 20; i++) {
    if (i >= lst.count()) break;
    d = lst[i].toInt();
    if (d > 3) d = 0;

    if (i == 0) {
      if (d == 0) {checkBoxHsl->setChecked(false);     checkBoxHslD->setChecked(false);}
      if (d == 1) {checkBoxHsl->setChecked(true);      checkBoxHslD->setChecked(true);}
    }
    if (i == 1) {
      if (d == 0) {checkBoxFsl->setChecked(false);     checkBoxFslD->setChecked(false);}
      if (d == 1) {checkBoxFsl->setChecked(true);      checkBoxFslD->setChecked(true);}
    }
    if (i == 2)  comboBoxLeh->setCurrentItem(d);
    if (i == 3)  comboBoxCeh->setCurrentItem(d);
    if (i == 4)  comboBoxReh->setCurrentItem(d);
    if (i == 5)  comboBoxLoh->setCurrentItem(d);
    if (i == 6)  comboBoxCoh->setCurrentItem(d);
    if (i == 7)  comboBoxRoh->setCurrentItem(d);
    if (i == 8)  comboBoxLef->setCurrentItem(d);
    if (i == 9)  comboBoxCef->setCurrentItem(d);
    if (i == 10) comboBoxRef->setCurrentItem(d);
    if (i == 11) comboBoxLof->setCurrentItem(d);
    if (i == 12) comboBoxCof->setCurrentItem(d);
    if (i == 13) comboBoxRof->setCurrentItem(d);

    if (i == 14) comboBoxIh->setCurrentItem(d);
    if (i == 15) comboBoxCh->setCurrentItem(d);
    if (i == 16) comboBoxOh->setCurrentItem(d);
    if (i == 17) comboBoxIf->setCurrentItem(d);
    if (i == 18) comboBoxCf->setCurrentItem(d);
    if (i == 19) comboBoxOf->setCurrentItem(d);
  }
}

// format of the string list:
// Pos 0: status of headsepline (0 = false, 1 = true)
// Pos 1: status of footsepline (0 = false, 1 = true)
// Pos 2--19: current items of combo boxes
QStringList HeadingsDialog::getOptions()
{
  QStringList lst;
  if (twoside) {
    if (checkBoxHslD->isChecked()) lst.push_back("1");
    else lst.push_back("0");
    if (checkBoxFslD->isChecked()) lst.push_back("1");
    else lst.push_back("0");
  }
  else {
    if (checkBoxHsl->isChecked()) lst.push_back("1");
    else lst.push_back("0");
    if (checkBoxFsl->isChecked()) lst.push_back("1");
    else lst.push_back("0");
  }

  QString s;
  lst.push_back(s.setNum(comboBoxLeh->currentItem()));
  lst.push_back(s.setNum(comboBoxCeh->currentItem()));
  lst.push_back(s.setNum(comboBoxReh->currentItem()));
  lst.push_back(s.setNum(comboBoxLoh->currentItem()));
  lst.push_back(s.setNum(comboBoxCoh->currentItem()));
  lst.push_back(s.setNum(comboBoxRoh->currentItem()));
  lst.push_back(s.setNum(comboBoxLef->currentItem()));
  lst.push_back(s.setNum(comboBoxCef->currentItem()));
  lst.push_back(s.setNum(comboBoxRef->currentItem()));
  lst.push_back(s.setNum(comboBoxLof->currentItem()));
  lst.push_back(s.setNum(comboBoxCof->currentItem()));
  lst.push_back(s.setNum(comboBoxRof->currentItem()));

  lst.push_back(s.setNum(comboBoxIh->currentItem()));
  lst.push_back(s.setNum(comboBoxCh->currentItem()));
  lst.push_back(s.setNum(comboBoxOh->currentItem()));
  lst.push_back(s.setNum(comboBoxIf->currentItem()));
  lst.push_back(s.setNum(comboBoxCf->currentItem()));
  lst.push_back(s.setNum(comboBoxOf->currentItem()));
  return lst;
}
