/***************************************************************************
                          marginprev.cpp  -  description
                             -------------------
    begin                : Mon Feb 23 2004
    copyright            : (C) 2004 by Nicola D�elin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "marginprev.h"
#include <qpainter.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qwmatrix.h>

MarginPrev::MarginPrev(QWidget *parent, const char *name ) : QFrame(parent,name) 
{
  #ifdef Q_OS_MACX
    color = QColor(200,200,200);
  #else
    color = backgroundColor();
  #endif

  wi = 210.0;
  he = 297.0;
  di = 5.0;
  bc = 0.0;
}

MarginPrev::~MarginPrev(){
}

void MarginPrev::setMargin(const float &w, const float &h, 
                           const float &div, const float &bcor)
{
  // these variables are globally defined
  wi = w;
  he = h;
  di = div;
  bc = bcor;

  float space = 10.0;
  float pages = 2.0;

  if (!twoside) {
    space = 0.0;
    pages = 1.0;
  }

  // these conditions will lead to divisions by 0:
  if (width() == (int)space) return;
  if (width() == 0) return;
  if (height() == 0) return;

  // make sure the paper preview fits in the widget (landscape / portrait)
  float divisor;
  if ((he / (float)height()) >= ((pages * wi) / float(width() - space))) divisor = he / (float)height();
  else divisor = (pages * wi) / float(width() - space);

  // will lead to divisions by 0:
  if (divisor == 0.0) return;

  // define the rectangle representing the page
  paper.setX((width() - int(wi / divisor))/2);
  paper.setY((height() - int(he / divisor))/2);
  paper.setWidth(int(wi / divisor));
  paper.setHeight(int(he / divisor));

  // define the rectangle representing the write area
  if (twoside) writeArea.setX(paper.x() + int((bc + 1.0 * float((wi-bc) / di)) / divisor));
  else writeArea.setX(paper.x() + int((bc + 1.5 * float((wi-bc) / di)) / divisor));

  writeArea.setY(paper.y() + int((he / di) / divisor));
  writeArea.setWidth(int((wi - bc - 3.0 * float((wi - bc) / di)) / divisor) + 1);
  writeArea.setHeight(int((he - 3*(he / di)) / divisor) + 1);

  // define the rectangle representing the BCOR
  if (bc > 0) {
    bcorRect.setX(paper.x()+1);
    bcorRect.setY(paper.top()+1);
    bcorRect.setWidth(int(bc / divisor));
    bcorRect.setHeight(paper.height() - 2);
  }

  QPainter p;
  QBrush fill(Qt::gray);
  fill.setStyle(DiagCrossPattern);
  QWMatrix m(-1.0, 0.0, 0.0, 1.0, 0.0, 0.0);;

  buffer.resize(width(), height());
  buffer.fill(color);

  // draw widget, but no calculation (all done before)
  p.begin(&buffer);
    if (twoside) {
      // draw the right page
      p.translate(((float)paper.width() + space)/2.0, 0.0);
      p.setPen(Qt::black);
      p.setBrush(Qt::white);
      p.drawRect(paper);
      p.setPen(Qt::gray);
      p.drawLine(writeArea.left(), paper.top()+1, writeArea.left(), paper.bottom()-1);
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.top(), paper.right()-1, writeArea.top());
      p.drawLine(writeArea.right(), paper.top()+1, writeArea.right(), paper.bottom()-1);
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.bottom(), paper.right()-1, writeArea.bottom());
      if (bc > 0) {
        // bcor line
        p.drawLine(bcorRect.right(), paper.top()+1, bcorRect.right(), paper.bottom()-1);
        p.setBrush(fill);
        p.setPen(NoPen);
        p.drawRect(bcorRect);
      }

      // invert and draw the left page
      p.setWorldMatrix(m);
      p.translate((((float)paper.width() + space)/2.0) - width(), 0.0 );
      p.setPen(Qt::black);
      p.setBrush(Qt::white);
      p.drawRect(paper);
      p.setPen(Qt::gray);
      p.drawLine(writeArea.left(), paper.top()+1, writeArea.left(), paper.bottom()-1);
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.top(), paper.right()-1, writeArea.top());
      p.drawLine(writeArea.right(), paper.top()+1, writeArea.right(), paper.bottom()-1);
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.bottom(), paper.right()-1, writeArea.bottom());
      if (bc > 0) {
        // bcor line
        p.drawLine(bcorRect.right(), paper.top()+1, bcorRect.right(), paper.bottom()-1);
        p.setBrush(fill);
        p.setPen(NoPen);
        p.drawRect(bcorRect);
      }
    }
    else {
      p.setPen(Qt::black);
      p.setBrush(Qt::white);
      p.drawRect(paper);
      p.setPen(Qt::gray);
      p.drawLine(writeArea.left(), paper.top()+1, writeArea.left(), paper.bottom()-1);
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.top(), paper.right()-1, writeArea.top());
      p.drawLine(writeArea.right(), paper.top()+1, writeArea.right(), paper.bottom()-1);
      p.drawLine(paper.left() + int(bc / divisor)+1, writeArea.bottom(), paper.right()-1, writeArea.bottom());
      if (bc > 0) {
        // bcor line
        p.drawLine(bcorRect.right(), paper.top()+1, bcorRect.right(), paper.bottom()-1);
        p.setBrush(fill);
        p.setPen(NoPen);
        p.drawRect(bcorRect);
      }
    }
  p.end();
  update();
}

void MarginPrev::clear()
{
  buffer.fill(color);
  update();
}

void MarginPrev::paintEvent(QPaintEvent *)
{
  bitBlt(this, 0, 0, &buffer);
}

void MarginPrev::resizeEvent(QResizeEvent *)
{
  buffer.resize(width(), height());
  setMargin(wi, he, di, bc);
}

void MarginPrev::setTwoside(bool b)
{
  twoside = b;
  setMargin(wi, he, di, bc);
}

