/***************************************************************************
                          settingsdialog.cpp  -  description
                             -------------------
    begin                : Son Dez 7 2003
    copyright            : (C) 2003 by Nicola Dbelin
    email                : adoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "settingsdialog.h"
#include <qfont.h>
#include <qfontdatabase.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qvaluelist.h>

SettingsDialog::SettingsDialog(QWidget *parent, const char *name) : SettingsDlg(parent,name) {

  listBoxFamily->insertStringList(fontDb.families());
  listBoxFamily->setCurrentItem(0);

}
SettingsDialog::~SettingsDialog(){
}

void SettingsDialog::familyChanged()
{
  QString w;
  font.setFamily(listBoxFamily->currentText());
  listBoxStyles->clear();
  listBoxPsize->clear();
  listBoxStyles->insertStringList(fontDb.styles(listBoxFamily->currentText()));
  font.setWeight(fontDb.weight(listBoxFamily->currentText(), listBoxStyles->currentText()));

  QValueList<int> vl = fontDb.smoothSizes(listBoxFamily->currentText(), listBoxStyles->currentText());
  listBoxPsize->clear();
  for (uint i = 0; i < vl.count(); i++)
    listBoxPsize->insertItem(w.number(vl[i]), -1);
  
  for (int i = 0; i < listBoxPsize->numRows(); i++)
    if (listBoxPsize->text(i) == w.number(font.pointSize()))
      listBoxPsize->setCurrentItem(i);

  lineEditExample->setFont(font);
}

void SettingsDialog::stylesChanged()
{
  font.setWeight(fontDb.weight(listBoxFamily->currentText(), listBoxStyles->currentText()));
  font.setItalic(fontDb.italic(listBoxFamily->currentText(), listBoxStyles->currentText()));
  font.setBold(fontDb.bold(listBoxFamily->currentText(), listBoxStyles->currentText()));
  lineEditExample->setFont(font);
}

void SettingsDialog::sizeChanged()
{
  font.setPointSize(spinBoxPsize->value());
  lineEditExample->setFont(font);
}

void SettingsDialog::newSize()
{
  spinBoxPsize->setValue(listBoxPsize->currentText().toInt());
}

QFont SettingsDialog::getFont() {return font;}
void SettingsDialog::setFont(QFont f)
{
  font = f;
  QString fam = font.toString().section(",", 0, 0);
  QString pts = font.toString().section(",", 1, 1);
  for (int i = 0; i < listBoxFamily->numRows(); i++)
    if (listBoxFamily->text(i) == fam) listBoxFamily->setCurrentItem(i);

  for (int i = 0; i < listBoxPsize->numRows(); i++)
    if (listBoxPsize->text(i) == pts) listBoxPsize->setCurrentItem(i);
}

void SettingsDialog::setWd(QString s) {lineEditWorkingDir->setText(s);}
QString SettingsDialog::getWd() {return lineEditWorkingDir->text();}
void SettingsDialog::setTd(QString s) {lineEditTexDir->setText(s);}
QString SettingsDialog::getTd() {return lineEditTexDir->text();}
void SettingsDialog::setLtd(QString s) {lineEditLTexDir->setText(s);}
QString SettingsDialog::getLtd() {return lineEditLTexDir->text();}


void SettingsDialog::wdChanged()
{
  lineEditWorkingDir->setText(QFileDialog::getExistingDirectory(lineEditWorkingDir->text()));
}

void SettingsDialog::tdChanged()
{
  lineEditTexDir->setText(QFileDialog::getExistingDirectory(lineEditTexDir->text()));
}

void SettingsDialog::ltdChanged()
{
  lineEditLTexDir->setText(QFileDialog::getExistingDirectory(lineEditLTexDir->text()));
}

void SettingsDialog::allSaved()
{
  QMessageBox::information(0, tr("Default settings saved"),
                           tr("Current settings were saved as defaults."));
}
