/***************************************************************************
                          texwiz.cpp  -  description
                             -------------------
    begin                : Sam Nov 29 16:47:18 CET 2003
    copyright            : (C) 2003 by Nicola D�elin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "texwiz.h"
#include "previewdialog.h"
#include "headingsdialog.h"
#include "pkgadddialog.h"
#include "settingsdialog.h"
#include "bibstyledialog.h"
#include "titledlg.h"
#include "marginprev.h"
#include "iconlistitem.h"
#include "img_files.h"
#include "markupdialog.h"
#include "templatedialog.h"

#ifdef USE_KDE
  #include <kapplication.h>
  #include <kfiledialog.h>
  #include <kmessagebox.h>
  #include <dcopclient.h>
#else
  #include <qfileinfo.h>
  #include <qmessagebox.h>
#endif

#include <qaction.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdatetime.h>
#include <qdatetimeedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qframe.h>
#include <qiconview.h>
#include <qinputdialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmenubar.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qsettings.h>
#include <qspinbox.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qtextcodec.h>
#include <qtextedit.h>
#include <qtextstream.h>
#include <qvaluelist.h>
#include <qwhatsthis.h>
#include <qwidgetstack.h>

#define VERSION "1.1.2a"

TexwizApp::TexwizApp()
{
  // setzt den Titel des Hauptfensters
  version = VERSION;
  setCaption(tr("TeXwiz " + version));

  templateList = new QPtrList<Template>();
  templateMenu = new QPopupMenu();
  MenuBar->insertItem(tr("&Templates"), templateMenu, -1, 2);

  // verbindet signale und slots die nicht im qtdesigner verbunden werden konnten
  connect(this, SIGNAL(sigPageStyle(int)), this, SLOT(slotShowScrhOptions(int)));
  connect(this, SIGNAL(sigResize()), this, SLOT(resizeListBox()));

  // setzt das datum des "Static Date" feldes auf heute
  dateEdit->setDate(QDate::currentDate());

  // initialisiert die bool'sche variable als "false". Wenn KOMA gebraucht wird,
  // wird sie spaeter auf "true" gesetzt, sonst bleibt sie "false", dito fuer
  // die anderen bools
  koma = false;
  pdf = false;
  twoside = false;
  listSort = true;
  idx = false;
  useKile = false;

  t_head = "";
  t_subject = "";
  t_publisher = "";
  t_dedication = "";
  tb_top = "";
  tb_bottom = "";
  outputFile = "";

  initListBox();
  initSettings();

  pushButtonScrhOptions->setEnabled(false); 
  textLabelNote->hide();
  marginChanged();
  emit sigPageStyle(comboBoxPageStyle->currentItem());

  // initialize DCOP communication if KDE support is enabled
  #ifdef USE_KDE
    client = kapp->dcopClient();
    bool done = client->attach();
    if (done) qDebug(tr("DCOP communication enabled."));
    else qDebug(tr("DCOP communication disabled."));
  #endif
}

TexwizApp::~TexwizApp()
{
}

// ------------------
// GUI Initialisation
// ------------------

void TexwizApp::initListBox()
{
// Listbox order is:
// Document Class
// Title and Author
// Page Layout
// TOC
// KOMA Script
// PDF Options (obsolete)
// Bibliography
// Optional Packages
// Letter Options (obsolete or temporarily deactivated)

  new IconListItem(listBox, qembed_findImage("edit"), tr("Document Class"));
  new IconListItem(listBox, qembed_findImage("identity"), tr("Title and Author"));
  new IconListItem(listBox, qembed_findImage("filefind"), tr("Page Layout"));
  new IconListItem(listBox, qembed_findImage("contents_toc"), tr("Lists"));
  new IconListItem(listBox, qembed_findImage("fonts"), tr("KOMA Script"));
//  new IconListItem(listBox, qembed_findImage("acroread"), tr("PDF Options"));
  new IconListItem(listBox, qembed_findImage("contents"), tr("Bibliography"));
  new IconListItem(listBox, qembed_findImage("package"), tr("Optional Packages"));
//  new IconListItem(listBox, qembed_findImage("mail_generic"), tr("Letter Options"));

  listBox->setCurrentItem(0);
}

// -------------------------
// Function implementations:
// -------------------------

QString TexwizApp::collectString()
{
  // der wird gebraucht um zahlen in strings umzuwandeln
  QString w;

  // Hier kommt der Code zum Abfragen der Einstellungen
  // --------------------------------------------------

  // Zuerst wird ein QString angelegt der den Text fuer
  // die Ausgabe sammelt, er ist vorerst leer ("null")
  QString output = QString::null;

  // Wichtige Felder werden auch in Strings eingelesen, damit
  // es nachher etwas uebersichtlicher bleibt ;-)
  title = textEditTitle->text();
  author = lineEditAuthor->text();

  // hier wird geschaut ob KOMA Skript verwendet werden soll.
  // die variable "koma" wurde als "false" initialisiert und
  // wird hier falls noetig auf "true" gesetzt. Alle Funktionen
  // hier kennen "koma" (ist global)
  if (checkBoxKoma->isChecked()) koma = true;
  else koma = false;
  if (comboBoxOutputFormat->currentItem() == 1) pdf = true;
  else pdf = false;
  if (comboBoxDuplex->currentItem() == 1) twoside = true;
  else twoside = false;
  if (checkBoxIndex->isChecked()) idx = true;
  else idx = false;
  if (checkBoxBiblio->isChecked()) biblio = true;
  else biblio = false;

  output += commentHeader();

  // Falls letter oder scrlettr als dokumentenklasse gewaehlt
  // ist wird die ausgabe in einer anderen funktion "collectLettr()"
  // zusammengestellt und zurueckgegeben, danach wird diese funktion
  // hier abgebrochen
//   if (comboBoxClass->currentItem() == 3) {
//     output += collectLetter();
//     return output;
//     }

  // die erstellung der zeile "documentclass" wurde in eine
  // eigene Funktion ausgelagert, da sie recht kompliziert ist.
  // sie liefert einen QString zurueck der die ganze Zeile enthaelt,
  // diese wird dem "Sammler"-String angehaengt, ebenso fuer pagestyle
  // und weitere funktionen
  output += documentClass(false);
  output += collectLang();
  output += usePackages();
  output += pageStyle();
  output += customMacros();
  if (pdf) output += pdfOptions();
  output += pageOptions();

  if (idx) output += "\\makeindex\n\n";

  output += "\\begin{document}\n\n";
  output += collectTitle();
  output += titlePage();
  output += "%--------------- Start of the document --------------\n\n\n\n";
  output += "%---------------- End of the document ---------------\n\n";

  if (biblio) {
    if (radioButtonBibtex->isOn()) {
      // use bibTeX with database
      output += "\\bibliographystyle{" + comboBoxBibSty->currentText() + "}\n";
      output += tr("% It is strongly recommended to convert the\n"
                   "% following absolute path to a relative path:\n");
      output += "\\bibliography{" + lineEditBibDb->text() + "}\n";
    }
    else {
      // use inline references
      output += "\\begin{thebibliography}{99}\n";
      output += "%  \\bibitem{}\n";
      output += "\\end{thebibliography}\n\n";
    }
  }
  if (idx) output += "\\printindex\n";
  output += "\\end{document}\n";

  // zuletzt wird der fertige LaTeX-String zurckgegeben, zum Beispiel zum Speichern
  // oder fuer das Vorschau-Fenster
  return output;
}


QString TexwizApp::collectLang()
{
  collector = QString::null;
  if (comboBoxLang->currentItem() == 1) collector += "\\usepackage{ngerman}\n";
  if (comboBoxLang->currentItem() == 2) collector += "\\usepackage[french]{babel}\n";

  if (comboBoxEncoding->currentItem() == 0) collector += "\\usepackage[latin1]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 1) collector += "\\usepackage[latin2]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 2) collector += "\\usepackage[latin3]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 3) collector += "\\usepackage[latin4]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 4) collector += "\\usepackage[latin5]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 5) collector += "\\usepackage[latin9]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 6) 
    collector += "\\usepackage{ucs}\n\\usepackage[utf8]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 7) collector += "\\usepackage[ansinew]{inputenc}\n";
  if (comboBoxEncoding->currentItem() == 8) collector += "\\usepackage[applemac]{inputenc}\n";

  return collector;
}

QString TexwizApp::commentHeader()
{
  collector  = "%----------------------------------------------------\n%\n";
  collector += "% Title:  " + title + "\n";
  collector += "% Author: " + author + "\n";
  collector += "% Date:   ";
  if (comboBoxLang->currentItem() == 0) collector += dateEdit->date().toString("MMMM d, yyyy");
  if (comboBoxLang->currentItem() == 1) collector += dateEdit->date().toString("d. MMMM yyyy");
  if (comboBoxLang->currentItem() == 2) collector += dateEdit->date().toString("d MMMM yyyy").lower();
  collector += "\n%\n%----------------------------------------------------\n";
  collector += "% Created with TeXwiz " + version + "\n";
  collector += "%----------------------------------------------------\n\n";

  return collector;
}

// diese Funktion stellt den Block 'Titel', 'Author' und 'Date' zusammen.
// die Formatierung des Datums ist sprachabh�gig
QString TexwizApp::collectTitle()
{
  collector = "";
  if (!title.isEmpty())  collector = "\n\\title{" + title + "}\n";
  // if maketitle is used, the title tag must be present, also if it's empty
  else {if (!radioButtonNTitle->isOn()) collector = "\n\\title{}\n";}
  if (!author.isEmpty()) collector += "\\author{" + author + "}\n";
  if (checkBoxStaticDate->isChecked()) {
    if (comboBoxLang->currentItem() == 0) date = dateEdit->date().toString("MMMM d, yyyy");
    if (comboBoxLang->currentItem() == 1) date = dateEdit->date().toString("d. MMMM yyyy");
    if (comboBoxLang->currentItem() == 2) date = dateEdit->date().toString("d MMMM yyyy").lower();
  }
  else date = "\\today";
  collector += "\\date{" + date + "}\n";
  if (koma && (radioButtonETitle->isOn())) {
    if (!t_extra.isEmpty())      collector += "\\extratitle{" + t_extra + "}\n";
    if (!t_head.isEmpty())       collector += "\\titlehead{" + t_head + "}\n";
    if (!t_subject.isEmpty())    collector += "\\subject{" + t_subject + "}\n";
    if (!t_publisher.isEmpty())  collector += "\\publishers{" + t_publisher + "}\n";
    if (!t_thanks.isEmpty())     collector += "\\thanks{" + t_thanks + "}\n";
    if (!t_dedication.isEmpty()) collector += "\\dedication{" + t_dedication + "}\n";
    if (comboBoxDuplex->currentItem() == 1) {
      if (!tb_top.isEmpty())     collector += "\\uppertitleback{" + tb_top + "}\n";
      if (!tb_bottom.isEmpty())  collector += "\\lowertitleback{" + tb_bottom + "}\n";
    }
    collector += "\n";
  }

  return collector;
}

// diese Funktion berechnet nur die zeile "documentclass" mit optionen
QString TexwizApp::documentClass(bool letter)
{
  // zwei Strings werden angelegt, einer fuer die ganze Zeile, der zweite
  // nur fuer die Optionen. Beide enthalten den Teil der sowieso am Anfang steht
  // ein weiterer Dummy-String w wird gebraucht um von nummern in strings umzuwandeln
  collector = "\\documentclass";
  QString options = "[";
  QString w;

  // optionen werden zusammengehaengt, die erste hat am Anfang kein Komma,
  // ALLE ANDEREN ABER SCHON
  if (comboBoxPaper->currentItem() == 0) options += "a4paper";
  if (comboBoxPaper->currentItem() == 1) options += "a5paper";
  if (comboBoxPaper->currentItem() == 2) options += "b5paper";
  if (comboBoxPaper->currentItem() == 3) options += "executivepaper";
  if (comboBoxPaper->currentItem() == 4) options += "legalpaper";
  if (comboBoxPaper->currentItem() == 5) options += "letterpaper";

  if (comboBoxFontSize->currentItem() == 0) options += ",\n               10pt";
  if (comboBoxFontSize->currentItem() == 1) options += ",\n               11pt";
  if (comboBoxFontSize->currentItem() == 2) options += ",\n               12pt";

  if (comboBoxOutputFormat->currentItem() == 1) options += ",\n               pdftex";

  if (comboBoxColumns->currentItem() == 1) options += ",\n               twocolumn";
  if (comboBoxDuplex->currentItem() == 1) options += ",\n               twoside";
  if (comboBoxOrient->currentItem() == 1) options += ",\n               landscape";

  if (comboBoxHeadingSize->currentItem() == 0) options += ",\n               smallheadings";
  if (comboBoxHeadingSize->currentItem() == 1) options += ",\n               normalheadings";
  if (comboBoxHeadingSize->currentItem() == 2) options += ",\n               bigheadings";

  if (koma && (checkBoxListsToc->isChecked()))         options += ",\n               liststotoc";
  if (koma && idx && (checkBoxIdxToc->isChecked()))    options += ",\n               idxtotoc";
  if (koma && biblio && (checkBoxBibToc->isChecked())) options += ",\n               bibtotoc";

  if (comboBoxPageStyle->currentItem() == 4) {
    if (headingOptions[0] == "1") options += ",\n               headsepline";
    if (headingOptions[1] == "1") options += ",\n               footsepline";
  }

  // spezielle KOMA optionen werden angehaengt
  if (koma) {
    options += ",\n               headinclude";
    options += ",\n               footexclude";
    // der DIV-Wert wird auf jeden Fall eingefuegt
    if (letter) options += ",\n               DIV=" + w.setNum(spinBoxDiv->value());
    else        options += ",\n               DIV" + w.setNum(spinBoxDiv->value());
    // der BCOR-Wert wird nur eingefuegt wenn er nicht 0 ist
    if (spinBoxBcor->value()) options += ",\n               BCOR" + w.setNum(spinBoxBcor->value()) + "mm";
    }

  // klammer der optionen schliessen
  options += "\n               ]";

  // falls keine Optionen vorhanden sind sollen keine leeren klammern eingefuegt
  // werden, sondern gar nichts, deshalb wird fuer diesen Fall "options" auf "null"
  // gesetzt  
  if (options == "[]") options = QString::null;

  // verfollstaendige die Zeile mit der Dokumentenklasse
  if (koma) {
    if (comboBoxClass->currentItem() == 0) collector += options + "{scrartcl}\n\n";
    if (comboBoxClass->currentItem() == 1) collector += options + "{scrreprt}\n\n";
    if (comboBoxClass->currentItem() == 2) collector += options + "{scrbook}\n\n";
    }
  else {
    if (comboBoxClass->currentItem() == 0) collector += options + "{article}\n\n";
    if (comboBoxClass->currentItem() == 1) collector += options + "{report}\n\n";
    if (comboBoxClass->currentItem() == 2) collector += options + "{book}\n\n";
    }

  if (comboBoxFontFam->currentItem() == 1) collector += "\\usepackage{mathptmx}\n"              // times
                                                        "\\usepackage[scaled=.90]{helvet}\n"
                                                        "\\usepackage{courier}\n";
  if (comboBoxFontFam->currentItem() == 2) collector += "\\usepackage{mathpazo}\n"              // palatino
                                                        "\\usepackage[scaled=.95]{helvet}\n"
                                                        "\\usepackage{courier}\n"
                                                        "\\linespread{1.05}\n";
  if (comboBoxFontFam->currentItem() == 3) collector += "\\usepackage{helvet}\n";
  if (comboBoxFontFam->currentItem() == 4) collector += "\\usepackage{avant}\n";
  if (comboBoxFontFam->currentItem() == 5) collector += "\\usepackage{bookman}\n";
  if (comboBoxFontFam->currentItem() == 6) collector += "\\usepackage{newcent}\n";
  if (idx && !letter) collector += "\\usepackage{makeidx}\n";

  // gibt die vollstaendige Zeile zurueck an den Caller
  return collector; 
}

// laedt alle packages aus der liste der pakete
QString TexwizApp::usePackages()
{
  collector = QString::null;
  bool calc = false,
       graph = false,
       space = false;

  for (int i = 0; i < listBoxPkgLoad->numRows(); i++) {
    collector += "\\usepackage{" + listBoxPkgLoad->text(i) + "}\n";
    if (listBoxPkgLoad->text(i) == "calc") calc = true;
    if (listBoxPkgLoad->text(i) == "graphicx") graph = true;
    if (listBoxPkgLoad->text(i) == "setspace") space = true;
  }

  // these two are always required, thus load if not done yet
  if (!calc) collector += "\\usepackage{calc}\n";
  if (!graph) collector += "\\usepackage{graphicx}\n";

  // handle the setspace packet
  QString lsp;
  if (comboBoxLSpacing->currentItem() == 0) lsp = "\\singlespacing\n";
  if (comboBoxLSpacing->currentItem() == 1) lsp = "\\onehalfspacing\n";
  if (comboBoxLSpacing->currentItem() == 2) lsp = "\\doublespacing\n";

  // if the package setspace was loaded manually,
  // always include the \...spacing line, else only
  // load the package if line spacing is > single
  if (space) collector += lsp;
  else if (comboBoxLSpacing->currentItem() != 0) {
    collector += "\\usepackage{setspace}\n";
    collector += lsp;
  }

  return collector;
}

// stellt den block fuer den pagestyle zusammen. Liest auch das
// options-fenster fuer scrheadings aus
QString TexwizApp::pageStyle()
{
  collector = "\n%---------------- Page style options ----------------\n\n";

  // einfache pagesyles brauchen nur eine Zeile...
  if (comboBoxPageStyle->currentItem() == 0) collector += "\\pagestyle{empty}\n";
  if (comboBoxPageStyle->currentItem() == 1) collector += "\\pagestyle{plain}\n";
  if (comboBoxPageStyle->currentItem() == 2) collector += "\\pagestyle{headings}\n";

  // fancy wird gleich speziell formatiert
  if (comboBoxPageStyle->currentItem() == 3) {
    collector += "\\usepackage{fancyhdr}\n\\pagestyle{fancy}\n";
    collector += "\\renewcommand{\\chaptermark}[1]{\\markboth{\\thechapter\\ #1}{}}\n";
    collector += "\\renewcommand{\\sectionmark}[1]{\\markright{\\thesection\\ #1}{}}\n";
    collector += "\\fancyhf{}\n\\fancyhead[L]{\\leftmark}\n\\fancyhead[R]{\\thepage}\n";
    }

  // fuer scrheadings muss der Options-dialog ausgelesen werden
  if (comboBoxPageStyle->currentItem() == 4) {
    QString idx2 = "\\pagemark";
    QString idx3 = "\\headmark";
    collector += "\\usepackage[automark]{scrpage2}\n\\pagestyle{scrheadings}\n";
    if (twoside) {
      // header:
      // left even
      if (headingOptions[2] == "1") collector += "\\lehead{}\n";
      if (headingOptions[2] == "2") collector += "\\lehead{" + idx2 + "}\n";
      if (headingOptions[2] == "3") collector += "\\lehead{" + idx3 + "}\n";
      // center even
      if (headingOptions[3] == "1") collector += "\\cehead{}\n";
      if (headingOptions[3] == "2") collector += "\\cehead{" + idx2 + "}\n";
      if (headingOptions[3] == "3") collector += "\\cehead{" + idx3 + "}\n";
      // right even
      if (headingOptions[4] == "1") collector += "\\rehead{}\n";
      if (headingOptions[4] == "2") collector += "\\rehead{" + idx2 + "}\n";
      if (headingOptions[4] == "3") collector += "\\rehead{" + idx3 + "}\n";
      // left odd
      if (headingOptions[5] == "1") collector += "\\lohead{}\n";
      if (headingOptions[5] == "2") collector += "\\lohead{" + idx2 + "}\n";
      if (headingOptions[5] == "3") collector += "\\lohead{" + idx3 + "}\n";
      // center odd
      if (headingOptions[6] == "1") collector += "\\cohead{}\n";
      if (headingOptions[6] == "2") collector += "\\cohead{" + idx2 + "}\n";
      if (headingOptions[6] == "3") collector += "\\cohead{" + idx3 + "}\n";
      // right odd
      if (headingOptions[7] == "1") collector += "\\rohead{}\n";
      if (headingOptions[7] == "2") collector += "\\rohead{" + idx2 + "}\n";
      if (headingOptions[7] == "3") collector += "\\rohead{" + idx3 + "}\n";

      // footer:
      // left even
      if (headingOptions[8] == "1") collector += "\\lefoot{}\n";
      if (headingOptions[8] == "2") collector += "\\lefoot{" + idx2 + "}\n";
      if (headingOptions[8] == "3") collector += "\\lefoot{" + idx3 + "}\n";
      // center even
      if (headingOptions[9] == "1") collector += "\\cefoot{}\n";
      if (headingOptions[9] == "2") collector += "\\cefoot{" + idx2 + "}\n";
      if (headingOptions[9] == "3") collector += "\\cefoot{" + idx3 + "}\n";
      // right even
      if (headingOptions[10] == "1") collector += "\\refoot{}\n";
      if (headingOptions[10] == "2") collector += "\\refoot{" + idx2 + "}\n";
      if (headingOptions[10] == "3") collector += "\\refoot{" + idx3 + "}\n";
      // left odd
      if (headingOptions[11] == "1") collector += "\\lofoot{}\n";
      if (headingOptions[11] == "2") collector += "\\lofoot{" + idx2 + "}\n";
      if (headingOptions[11] == "3") collector += "\\lofoot{" + idx3 + "}\n";
      // center odd
      if (headingOptions[12] == "1") collector += "\\cofoot{}\n";
      if (headingOptions[12] == "2") collector += "\\cofoot{" + idx2 + "}\n";
      if (headingOptions[12] == "3") collector += "\\cofoot{" + idx3 + "}\n";
      // right odd
      if (headingOptions[13] == "1") collector += "\\rofoot{}\n";
      if (headingOptions[13] == "2") collector += "\\rofoot{" + idx2 + "}\n";
      if (headingOptions[13] == "3") collector += "\\rofoot{" + idx3 + "}\n";
    }
    else {
      // header:
      // inner
      if (headingOptions[14] == "1") collector += "\\ihead{}\n";
      if (headingOptions[14] == "2") collector += "\\ihead{" + idx2 + "}\n";
      if (headingOptions[14] == "3") collector += "\\ihead{" + idx3 + "}\n";
      // centre
      if (headingOptions[15] == "1") collector += "\\chead{}\n";
      if (headingOptions[15] == "2") collector += "\\chead{" + idx2 + "}\n";
      if (headingOptions[15] == "3") collector += "\\chead{" + idx3 + "}\n";
      // outer
      if (headingOptions[16] == "1") collector += "\\ohead{}\n";
      if (headingOptions[16] == "2") collector += "\\ohead{" + idx2 + "}\n";
      if (headingOptions[16] == "3") collector += "\\ohead{" + idx3 + "}\n";

      // footer:
      // inner
      if (headingOptions[17] == "1") collector += "\\ifoot{}\n";
      if (headingOptions[17] == "2") collector += "\\ifoot{" + idx2 + "}\n";
      if (headingOptions[17] == "3") collector += "\\ifoot{" + idx3 + "}\n";
      // centre
      if (headingOptions[18] == "1") collector += "\\cfoot{}\n";
      if (headingOptions[18] == "2") collector += "\\cfoot{" + idx2 + "}\n";
      if (headingOptions[18] == "3") collector += "\\cfoot{" + idx3 + "}\n";
      // outer
      if (headingOptions[19] == "1") collector += "\\ofoot{}\n";
      if (headingOptions[19] == "2") collector += "\\ofoot{" + idx2 + "}\n";
      if (headingOptions[19] == "3") collector += "\\ofoot{" + idx3 + "}\n";
    }
  }

  collector += "\n";
  if (koma) {
    if (comboBoxKCaptions->currentItem() == 0) collector += "\\setcaphanging\n\n";
    if (comboBoxKCaptions->currentItem() == 1) collector += "\\setcapindent{1em}\n\n";
    if (comboBoxKCaptions->currentItem() == 2) collector += "\\setcapindent*{1em}\n\n";
    if (comboBoxKCaptions->currentItem() == 3) collector += "\\setcapindent{-1em}\n\n";
  }
  return collector;
}

/* collects page options like floatpagefraction and sloppy */
QString TexwizApp::pageOptions()
{
  // reset the return string
  collector = "";

  // the usual dummy string
  QString w;
  if (spinBoxTocDepth->value() > 0) 
    collector += "\\setcounter{tocdepth}{" + w.setNum(spinBoxTocDepth->value()) + "}\n";
  if (spinBoxFpf->value() > 0) {
    collector += "\\renewcommand{\\floatpagefraction}{0." + w.setNum(spinBoxFpf->value()) + "}\n";
    collector += "\\renewcommand{\\dblfloatpagefraction}{0." + w.setNum(spinBoxFpf->value()) + "}\n";
    }
  if (checkBoxSloppy->isChecked()) {
    collector += "\n% sloppy wordspacing:\n";
    collector += "\\tolerance 1414\n";
    collector += "\\hbadness 1414\n";
    collector += "\\emergencystretch 1.5em\n";
    collector += "\\hfuzz 0.3pt\n";
    collector += "\\widowpenalty=10000\n";
    collector += "\\vfuzz \\hfuzz\n";
    collector += "\\raggedbottom\n\n";
    }

  return collector;
}

QString TexwizApp::pdfOptions()
{
  collector  = "%---------------- PDF output options ----------------\n\n";
  collector += "\\pdfoutput=1\n\\usepackage{thumbpdf}\n\\usepackage[bookmarks]{hyperref}\n";
  collector += "\\pdfimageresolution=300\n";
  collector += "\\pdfcompresslevel=1\n\n";

  collector += "\\pdfinfo{\n";
  collector += "/Title(" + title + ")\n";
  collector += "/Creator(TeX)\n";
  collector += "/Producer(PdfTeX)\n";
  collector += "/Author(" + author + ")\n";
  collector += "/CreationDate(D:" + QDateTime::currentDateTime().toString("yyyyMMddhhmmss") + ")\n";
  collector += "/Subject(" + title + ")}\n\n";
  collector += "%----------------------------------------------------\n\n";

  return collector;
}

QString TexwizApp::customMacros()
{
  if (!checkBoxMarkups->isChecked()) return QString::null;
  collector  = "%-------------- Custom Markup Commands --------------\n\n";
  collector += customMarkups.join("\n");
  collector += "\n\n";
  return collector;
}

QString TexwizApp::titlePage()
{
  collector = QString::null;
  if (radioButtonTitle->isChecked() || radioButtonPTitle->isChecked() || radioButtonETitle->isChecked())
    collector += "\\maketitle\n";
  if (radioButtonCTitle->isChecked()) {
    collector += "%----------------- Custom title page ----------------\n\n";
    collector += "\\begin{titlepage}\n"
                 "  \\vspace*{5cm}\n"
                 "  \\Huge\n"
                 "  \\noindent\n"
                 "  \\textsl{\\textbf{" + title + "}}\\\\\n"
                 "  \\rule{\\textwidth}{1.5mm}\n"
                 "  \\begin{flushright}\n"
                 "  \\Large\n"
                 "  \\textsl{\\textbf{" + author + "}}\\\\\n"
                 "  \\large\n"
                 "  \\textsl{" + date + "}\n"
                 "  \\end{flushright}\n"
                 "  \\vfill\n"
                 "  \\normalsize\n"
                 "\\end{titlepage}\n";
    }

  if (radioButtonPTitle->isChecked() || radioButtonCTitle->isChecked()) {
    if (comboBoxClass->currentItem() == 0) collector += "\\newpage\n";
  }

  if (checkBoxToc->isChecked()) collector += "\\tableofcontents\n\n";
  if (checkBoxLof->isChecked()) collector += "\\listoffigures\n\n";
  if (checkBoxLot->isChecked()) collector += "\\listoftables\n\n";

  return collector;
}


void TexwizApp::setKile(bool b) 
{
  useKile = b;
  if (useKile) {
    fileSaveAsAction->setMenuText(tr("Save and open in Kile..."));
    fileSaveAsAction->setStatusTip(tr("Save and open in Kile..."));
    fileSaveAsAction->setToolTip(tr("Save and open in Kile"));
    MenuBar->hide();
    statusBar()->setShown(false);
    toolBar->setShown(false);
  }
  else {
    frameDialog->hide();
  }
}

// ---------------------
// Slot implementations:
// ---------------------

void TexwizApp::fileSaveAs()
{
  bool exists = true;
  QFileInfo finfo;
  QString filename;
  QString path = workingDir;
  QFile file;

  while (exists) {
    #ifdef USE_KDE
      filename = KFileDialog::getSaveFileName(path, "*.tex|LaTeX source file (*.tex)", this);
    #else
      filename = QFileDialog::getSaveFileName(path, "*.tex *.TEX");
    #endif
    if (filename.isEmpty()) return;

    finfo.setFile(filename);

    // if no extension ".tex" is added, TeXwiz will add it:
    if (finfo.extension(false).lower() != "tex") filename += ".tex";
    finfo.setFile(filename);
    file.setName(filename);

    if (file.exists()) {
      exists = true;
      #ifdef USE_KDE
        if (KMessageBox::questionYesNo(0, tr("File " + filename + " exists.\nDo you want to overwrite it?"), 
	                              tr("File exists")) == KMessageBox::Yes)
      #else
        if (!QMessageBox::information(0, tr("File exists"), tr("File " + filename + " exists.\nDo you want to overwrite it?"),
                                      tr("&Yes"), tr("&No")))
      #endif
        exists = false;
      }
    else exists = false;
  }

  bool _kile = false;
  #ifdef USE_KDE
    if (useKile) _kile = true;
  #endif

  // in kile mode
  if (_kile) {
    #ifdef USE_KDE  // that's necessary again, because some variables are defined only in this mode
    file.setName(filename);

    if (!file.open(IO_WriteOnly)) {
      KMessageBox::error(0, tr("Couldn't save file\n") + filename, tr("Save failed"));
      return;
    }


    QTextStream s(&file);
    int cd = comboBoxEncoding->currentItem();
    bool found = false;
    QTextCodec *codec;
    codec = QTextCodec::codecForName("Latin1");

    if (cd == 0) {
      s.setEncoding(QTextStream::Latin1);
      found = true;
    }

    if (cd == 1) codec = QTextCodec::codecForName("ISO8859-2");
    if (cd == 2) codec = QTextCodec::codecForName("ISO8859-3");
    if (cd == 3) codec = QTextCodec::codecForName("ISO8859-4");
    if (cd == 4) codec = QTextCodec::codecForName("ISO8859-5");
    if (cd == 5) codec = QTextCodec::codecForName("ISO8859-15");

    if (cd == 6) {
      s.setEncoding(QTextStream::UnicodeUTF8);
      found = true;
    }

    if (cd == 7) codec = QTextCodec::codecForName("CP1252");
    if (cd == 8) codec = QTextCodec::codecForName("Apple Roman");

    if (!found) {
      s.setEncoding(QTextStream::Locale);
    }

    s << collectString();
    file.close();

    // if using KDE, open the file in kile
    client->send("kile",
                 "Kile",
                 "fileSelected(QString)",
                 filename);
    #endif
  }
  else {
    if (!file.open(IO_WriteOnly)) {
      #ifdef USE_KDE
        KMessageBox::error(this, tr("Couldn't save file\n") + filename, tr("Save failed"));
      #else
        QMessageBox::warning(this, tr("Save failed"), tr("Couldn't save file\n") + filename);
      #endif
      return;
    }

    QTextStream s(&file);
    s << collectString();
    file.close();
  }

  if (_kile) close();
}

void TexwizApp::filePreview()
{
  PreviewDialog *prev = new PreviewDialog();
  prev->setFont(prevFont);
  prev->setText(collectString());
  if (restoreSize) prev->resize(prev_w, prev_h);
  if (restorePos) prev->move(prev_x, prev_y);

  // the dialog can only be closed by rejecting it, thus settings have to
  // be saved in this case
  if (prev->exec() == QDialog::Rejected) {
    prev_w = prev->width();
    prev_h = prev->height();
    prev_x = prev->x();
    prev_y = prev->y();
    }
  delete prev;
}

void TexwizApp::resizeEvent(QResizeEvent *) {emit sigResize();}
void TexwizApp::showEvent(QShowEvent *) {emit sigResize();}

void TexwizApp::closeEvent(QCloseEvent *ce)
{
//  if (!QMessageBox::question(this, tr("Exit TeXwiz"),
//                             tr("Do you really want to quit TeXwiz?"),
//                             tr("&Yes"), tr("&No"))) {
  saveSettings();
  ce->accept();
//  }
  return;
}

void TexwizApp::helpAbout()
{
  #ifdef USE_KDE
    KMessageBox::about(this,
                       tr("TeXwiz Version " + version + "\n(c) 2003-2005 by Nicola and Andri Doebelin"),
		       tr("About...") );
  #else
    QMessageBox::about(this, tr("About..."),
                       tr("TeXwiz\nVersion " + version + "\n(c) 2003-2005 by Nicola and Andri Doebelin"));
  #endif
}

void TexwizApp::toggleKoma(bool b)
{
  frameKoma->setEnabled(b);
  if (b) {if (radioButtonETitle->isOn()) pushButtonETitle->setEnabled(b);}
  else 
  {
    pushButtonETitle->setEnabled(b);
    if (radioButtonETitle->isOn()) radioButtonNTitle->setChecked(true);
  }
  radioButtonETitle->setEnabled(b);
  groupBoxToc->setEnabled(b);  
}

void TexwizApp::updatePreview()
{
  int i = comboBoxDuplex->currentItem();
  if (i == 0) marginPrev->setTwoside(false);
  else marginPrev->setTwoside(true);
  marginChanged();
}

void TexwizApp::extendedTitles()
{
  TitleDlg *tdlg = new TitleDlg();

  tdlg->editTitleHead->setText(t_head);
  tdlg->editSubject->setText(t_subject);
  tdlg->editPublisher->setText(t_publisher);
  tdlg->editThanks->setText(t_thanks);
  tdlg->editDedication->setText(t_dedication);
  tdlg->editExtra->setText(t_extra);
  tdlg->editBsTop->setText(tb_top);
  tdlg->editBsBottom->setText(tb_bottom);

  if (comboBoxDuplex->currentItem() == 0) tdlg->tabWidget->setTabEnabled(tdlg->tabWidget->page(2), false);
  else tdlg->tabWidget->setTabEnabled(tdlg->tabWidget->page(2), true);

  if (tdlg->exec() == QDialog::Accepted) {
    t_head       = tdlg->editTitleHead->text();
    t_subject    = tdlg->editSubject->text();
    t_publisher  = tdlg->editPublisher->text();
    t_thanks     = tdlg->editThanks->text();
    t_dedication = tdlg->editDedication->text();
    t_extra      = tdlg->editExtra->text();
    tb_top       = tdlg->editBsTop->text();
    tb_bottom    = tdlg->editBsBottom->text();
  }

  delete tdlg;
}

void TexwizApp::editCustomMarkup()
{
  MarkupDialog *mdlg = new MarkupDialog();
  mdlg->setList(customMarkups);

  if (mdlg->exec() == QDialog::Accepted) customMarkups = mdlg->getList();

  delete mdlg;
}

void TexwizApp::slotShowScrhOptions(int a)
{
  if (a == 4) {
    pushButtonScrhOptions->setEnabled(true);
    textLabelNote->show();
    }
  else {
    pushButtonScrhOptions->setEnabled(false);
    textLabelNote->hide();
    }
}

void TexwizApp::headingsOptions()
{
 HeadingsDialog *hdlg = new HeadingsDialog();

  if (comboBoxDuplex->currentItem() == 0) hdlg->setTwoside(false);
  else hdlg->setTwoside(true);
  hdlg->setOptions(headingOptions);

  if (hdlg->exec() == QDialog::Accepted) {
    headingOptions = hdlg->getOptions();
  }
  delete hdlg;
}

void TexwizApp::marginChanged()
{
  QString s;
  float div = (float)spinBoxDiv->value();
  float bcor = (float)spinBoxBcor->value();
  float w = 210.0, h = 297.0;
  float i = 0.0, o = 0.0, t = 0.0, b = 0.0;
  if (comboBoxPaper->currentItem() == 0) {w = 210.0;  h = 297.0;}
  if (comboBoxPaper->currentItem() == 1) {w = 148.0;  h = 210.0;}
  if (comboBoxPaper->currentItem() == 2) {w = 176.0;  h = 250.0;}
  if (comboBoxPaper->currentItem() == 3) {w = 184.15; h = 266.7;}
  if (comboBoxPaper->currentItem() == 4) {w = 215.9;  h = 355.6;}
  if (comboBoxPaper->currentItem() == 5) {w = 215.9;  h = 279.4;}

  //check for landscape and swap w and h if necessary
  if (comboBoxOrient->currentItem() == 1) {
    float x = w;
    w = h;
    h = x;
  }

  if (comboBoxDuplex->currentItem() == 0) {
    i = bcor + 1.5 * ((w - bcor) / div);
    o = 1.5 * (w - bcor) / div;
  }
  else {
    i = bcor + 1.0 * ((w - bcor) / div);
    o = 2.0 * (w - bcor) / div;
  }

  t = h / div;
  b = 2.0 * h / div;

  marginPrev->setMargin(w, h, div, bcor);

  labelMarginTop->setText(s.sprintf("%.1f mm", t));
  labelMarginBottom->setText(s.sprintf("%.1f mm", b));
  labelMarginInner->setText(s.sprintf("%.1f mm", i));
  labelMarginOuter->setText(s.sprintf("%.1f mm", o));  
}

void TexwizApp::pkgIns()
{
  QStringList s;
  int ci = listBoxPkgAvail->currentItem();

  for (int i = 0; i < (int)listBoxPkgAvail->count(); i++)
    if (listBoxPkgAvail->isSelected(i))
      s.append(listBoxPkgAvail->text(i));

  for (int i = (int)listBoxPkgAvail->count()-1; i >= 0; i--)
    if (listBoxPkgAvail->isSelected(i))
      listBoxPkgAvail->removeItem(i);

  listBoxPkgLoad->insertStringList(s, -1);

  listBoxPkgAvail->setCurrentItem(ci);
  listBoxPkgAvail->setSelected(ci, true);
}

void TexwizApp::pkgRem()
{
  QStringList s;

  for (int i = 0; i < (int)listBoxPkgLoad->count(); i++)
    if (listBoxPkgLoad->isSelected(i))
      s.append(listBoxPkgLoad->text(i));

  for (int i = (int)listBoxPkgLoad->count()-1; i >= 0; i--)
    if (listBoxPkgLoad->isSelected(i))
      listBoxPkgLoad->removeItem(i);

  listBoxPkgAvail->insertStringList(s, -1);

  listBoxPkgLoad->setCurrentItem(0);
  listBoxPkgLoad->setSelected(0, true);
}

void TexwizApp::pkgAdd()
{
  PkgAddDialog *pad = new PkgAddDialog();
  pad->setDirs(texDir, ltexDir);
  QString w;
  if (pad->exec() == QDialog::Accepted)
    if (pad->checkBox->isChecked()) {
      if (pad->listBox->count()) {          // not empty
        listBoxPkgAvail->clear();
        for (int i = 0; i < (int)pad->listBox->count(); i++) 
          listBoxPkgAvail->insertItem(pad->listBox->text(i), -1);
        }
      }
    else
      if (!pad->lineEdit->text().isEmpty())
        listBoxPkgAvail->insertItem(pad->lineEdit->text(), -1);
  delete pad;
}

void TexwizApp::pkgDel()
{
  #ifdef USE_KDE
    if (KMessageBox::questionYesNo(this, tr("Do you want to clear the selected items?"), tr("Clear items")) == KMessageBox::No)
       return;
  #else
    if (QMessageBox::information(this, tr("Clear items"), tr("Do you want to clear\nthe selected items?"),
                               tr("&Yes"), tr("&No"))) return;
  #endif
  
  int c = listBoxPkgAvail->currentItem();
  for (int i = (int)listBoxPkgAvail->count()-1; i >= 0; i--)
    if (listBoxPkgAvail->isSelected(i))
      listBoxPkgAvail->removeItem(i);

  if (c >= (int)listBoxPkgAvail->count()) c = listBoxPkgAvail->count() - 1;
  listBoxPkgAvail->setCurrentItem(c);
}

void TexwizApp::pkgClear()
{
  for (int i = listBoxPkgLoad->numRows() - 1; i >= 0; i--) {
    listBoxPkgAvail->insertItem(listBoxPkgLoad->text(i), 0);
    listBoxPkgLoad->removeItem(i);
    }
}

void TexwizApp::pkgSort()
{
  listBoxPkgAvail->sort(listSort);
  listSort = !listSort;
}

void TexwizApp::toggleToolbar(bool b) {toolBar->setShown(b);}
void TexwizApp::toggleStatusbar(bool b) {statusBar()->setShown(b);}

void TexwizApp::initSettings()
{
  QSettings *settings = new QSettings();
  QString avail, loaded, font, bibstyles;

  settings->beginGroup("/texwiz");
    settings->beginGroup("/geometry");
      int w = settings->readNumEntry("/width", 500);
      int h = settings->readNumEntry("/height", 300);
      int x = settings->readNumEntry("/x_pos", 0);
      int y = settings->readNumEntry("/y_pos", 0);
      #ifdef Q_OS_MACX
        if (y < 10) y = 10;
      #endif
      prev_w = settings->readNumEntry("/preview_width", 400);
      prev_h = settings->readNumEntry("/preview_height", 300);
      prev_x = settings->readNumEntry("/preview_x", 100);
      prev_y = settings->readNumEntry("/preview_y", 100);
    settings->endGroup();
    settings->beginGroup("/preferences");
      font = settings->readEntry("/font_prevdlg", "Courier [Adobe],10,-1,5,50,0,0,0,0,0");
      workingDir = settings->readEntry("/workingdir", QString::null);
      texDir = settings->readEntry("/texdir", QString::null);
      ltexDir = settings->readEntry("/localtexdir", QString::null);
      lineEditBibDb->setText(settings->readEntry("/bibtex_db", QString::null));
      bool sb = settings->readBoolEntry("/statusbar", true);
      bool tb = settings->readBoolEntry("/toolbar", true);
      restoreSize = settings->readBoolEntry("/restore_Size", true);
      restorePos = settings->readBoolEntry("/restore_Pos", true);
      oldTemplates = settings->readListEntry("/template_names");
    settings->endGroup();
    settings->beginGroup("/tex_options");
      checkBoxStaticDate->setChecked(settings->readBoolEntry("/staticDate", false));
      buttonGroupTitle->setButton(settings->readNumEntry("/id_buttonTitle", 0));
      comboBoxEncoding->setCurrentItem(settings->readNumEntry("/encoding", 0));
      checkBoxToc->setChecked(settings->readBoolEntry("/toc", false));
      groupBoxToc->setEnabled(settings->readBoolEntry("/toc", false));
      checkBoxLof->setChecked(settings->readBoolEntry("/lof", false));
      checkBoxLot->setChecked(settings->readBoolEntry("/lot", false));
      checkBoxIndex->setChecked(settings->readBoolEntry("/idx", false));
      checkBoxBiblio->setChecked(settings->readBoolEntry("/bib", false));
      comboBoxClass->setCurrentItem(settings->readNumEntry("/dclass", 0));
      comboBoxPaper->setCurrentItem(settings->readNumEntry("/paper", 0));
      comboBoxFontSize->setCurrentItem(settings->readNumEntry("/fontsize", 0));
      comboBoxFontFam->setCurrentItem(settings->readNumEntry("/fontfam", 0));
      comboBoxOutputFormat->setCurrentItem(settings->readNumEntry("/format", 0));
      comboBoxLang->setCurrentItem(settings->readNumEntry("/language", 0));
      bool use_koma = settings->readBoolEntry("/use_koma", true);
      comboBoxDuplex->setCurrentItem(settings->readNumEntry("/duplex", 0));
      comboBoxColumns->setCurrentItem(settings->readNumEntry("/columns", 0));
      comboBoxOrient->setCurrentItem(settings->readNumEntry("/orientation", 0));
      comboBoxPageStyle->setCurrentItem(settings->readNumEntry("/pagestyle", 0));
      buttonGroupBibOpt->setButton(settings->readNumEntry("/bib_inline", 0));
      spinBoxTocDepth->setValue(settings->readNumEntry("/toc_depth", 0));
      checkBoxListsToc->setChecked(settings->readBoolEntry("/lists_to_toc", false));
      checkBoxIdxToc->setChecked(settings->readBoolEntry("/idx_to_toc", false));
      checkBoxBibToc->setChecked(settings->readBoolEntry("/bib_to_toc", false));
      spinBoxFpf->setValue(settings->readNumEntry("/float_page_fraction", 0));
      checkBoxSloppy->setChecked(settings->readBoolEntry("/sloppy", false));
      comboBoxLSpacing->setCurrentItem(settings->readNumEntry("/lspacing", 0));
      comboBoxKCaptions->setCurrentItem(settings->readNumEntry("/captions", 0));
      checkBoxMarkups->setChecked(settings->readBoolEntry("/use_custom_macros", false));
      headingOptions = settings->readListEntry("/heading_options");
      customMarkups = settings->readListEntry("/custom_macros");
    settings->endGroup();
    settings->beginGroup("/packages");
      avail = settings->readEntry("/pkgs", QString::null);
      loaded = settings->readEntry("/pkgs_loaded", QString::null);
      bibstyles = settings->readEntry("/bib_styles", QString::null);
      int bs = settings->readNumEntry("/bib_current_style", 0);
    settings->endGroup();

    settings->beginGroup("/templates");
      templateList->clear();
      QStringList entry;
      QStringList::iterator it = oldTemplates.begin();

      while (it != oldTemplates.end()) {
        entry = settings->readListEntry(*it);
        Template *t = new Template();
        t->setFromLongStringList(entry);
        templateList->append(t);
        ++it;
      }
    settings->endGroup();
  settings->endGroup();

  if (restoreSize) resize(w, h);
  if (restorePos) move(x, y);

  checkBoxKoma->setChecked(use_koma);
  toggleKoma(use_koma);

  while (headingOptions.size() < 20) headingOptions.push_back("0");

  listBoxPkgAvail->clear();
  listBoxPkgLoad->clear();
  comboBoxBibSty->clear();
  listBoxPkgAvail->insertStringList(QStringList::split(";", avail, false), -1);
  listBoxPkgLoad->insertStringList(QStringList::split(";", loaded, false), -1);
  comboBoxBibSty->insertStringList(QStringList::split(";", bibstyles, false), -1);
  frameBibtex->setEnabled(checkBoxBiblio->isChecked());
  groupBoxBibtex->setEnabled(radioButtonBibtex->isOn());

  if (comboBoxDuplex->currentItem() == 0) marginPrev->setTwoside(false);
  if (comboBoxDuplex->currentItem() == 1) marginPrev->setTwoside(true);

  comboBoxBibSty->setCurrentItem(bs);
  prevFont.fromString(font);

  statusbarAction->setOn(sb);
  toolbarAction->setOn(tb);
  statusBar()->setShown(sb);
  toolBar->setShown(tb);
  updateTemplates();

  delete settings;
}

void TexwizApp::saveSettings(bool all)
{
  QString avail, bibstyles, loaded;

  for (int i = 0; i < listBoxPkgAvail->numRows(); i++)
    avail += listBoxPkgAvail->text(i) + ";";
  for (int i = 0; i < listBoxPkgLoad->numRows(); i++)
    loaded += listBoxPkgLoad->text(i) + ";";
  for (int i = 0; i < comboBoxBibSty->count(); i++)
    bibstyles += comboBoxBibSty->text(i) + ";";

  QStringList tmpl;
  Template *t;
  for (t = templateList->first(); t; t = templateList->next()) {
    tmpl.append("/" + t->name());
  }

  QSettings *settings = new QSettings();
  settings->beginGroup("/texwiz");
    settings->beginGroup("/geometry");
      settings->writeEntry("/width", width());
      settings->writeEntry("/height", height());
      settings->writeEntry("/x_pos", x());
      settings->writeEntry("/y_pos", y());
      settings->writeEntry("/preview_width", prev_w);
      settings->writeEntry("/preview_height", prev_h);
      settings->writeEntry("/preview_x", prev_x);
      settings->writeEntry("/preview_y", prev_y);
    settings->endGroup();
    settings->beginGroup("/preferences");
      settings->writeEntry("/font_prevdlg", prevFont.toString());
      settings->writeEntry("/workingdir", workingDir);
      settings->writeEntry("/texdir", texDir);
      settings->writeEntry("/localtexdir", ltexDir);
      settings->writeEntry("/bibtex_db", lineEditBibDb->text());
    if (!useKile) {
      settings->writeEntry("/statusbar", statusBar()->isShown());
      settings->writeEntry("/toolbar", toolBar->isShown());
    }
      settings->writeEntry("/restore_Size", restoreSize);
      settings->writeEntry("/restore_Pos", restorePos);
      settings->writeEntry("/template_names", tmpl);
    settings->endGroup();
    settings->beginGroup("/tex_options");
      if (all) {
        settings->writeEntry("/staticDate", checkBoxStaticDate->isChecked());
        settings->writeEntry("/id_buttonTitle", buttonGroupTitle->selectedId());
        settings->writeEntry("/encoding", comboBoxEncoding->currentItem());
        settings->writeEntry("/toc", checkBoxToc->isChecked());
        settings->writeEntry("/lof", checkBoxLof->isChecked());
        settings->writeEntry("/lot", checkBoxLot->isChecked());
        settings->writeEntry("/idx", checkBoxIndex->isChecked());
        settings->writeEntry("/bib", checkBoxBiblio->isChecked());
        settings->writeEntry("/dclass", comboBoxClass->currentItem());
        settings->writeEntry("/paper", comboBoxPaper->currentItem());
        settings->writeEntry("/fontsize", comboBoxFontSize->currentItem());
        settings->writeEntry("/fontfam", comboBoxFontFam->currentItem());
        settings->writeEntry("/format", comboBoxOutputFormat->currentItem());
        settings->writeEntry("/language", comboBoxLang->currentItem());
        settings->writeEntry("/use_koma", checkBoxKoma->isChecked());
        settings->writeEntry("/duplex", comboBoxDuplex->currentItem());
        settings->writeEntry("/columns", comboBoxColumns->currentItem());
        settings->writeEntry("/orientation", comboBoxOrient->currentItem());
        settings->writeEntry("/pagestyle", comboBoxPageStyle->currentItem());
        settings->writeEntry("/bib_inline", buttonGroupBibOpt->selectedId());
        settings->writeEntry("/toc_depth", spinBoxTocDepth->value());
        settings->writeEntry("/lists_to_toc", checkBoxListsToc->isChecked());
        settings->writeEntry("/idx_to_toc", checkBoxIdxToc->isChecked());
        settings->writeEntry("/bib_to_toc", checkBoxBibToc->isChecked());
        settings->writeEntry("/float_page_fraction", spinBoxFpf->value());
        settings->writeEntry("/sloppy", checkBoxSloppy->isChecked());
        settings->writeEntry("/lspacing", comboBoxLSpacing->currentItem());
        settings->writeEntry("/captions", comboBoxKCaptions->currentItem());
        settings->writeEntry("/use_custom_macros", checkBoxMarkups->isChecked());
        settings->writeEntry("/heading_options", headingOptions);
      }
      settings->writeEntry("/custom_macros", customMarkups);
    settings->endGroup();
    settings->beginGroup("/packages");
      settings->writeEntry("/pkgs", avail);
      settings->writeEntry("/pkgs_loaded", loaded);
      settings->writeEntry("/bib_styles", bibstyles);
      settings->writeEntry("/bib_current_style", comboBoxBibSty->currentItem());
    settings->endGroup();
    settings->beginGroup("/templates");
      // remove old template entries, otherwise they may accumulate when renamed
      QStringList::iterator it = oldTemplates.begin();
      while (it != oldTemplates.end()) {
        settings->removeEntry(*it);
        it++;
      }

      // write new templates
      Template *tp;
      for (tp = templateList->first(); tp; tp = templateList->next()) {
        QString lname = "/" + tp->name();
        settings->writeEntry(lname, tp->writeToStringList());
      }
    settings->endGroup();
  settings->endGroup();
  delete settings;
}

void TexwizApp::saveAllSettings() {saveSettings(true);}

void TexwizApp::preferences()
{
  SettingsDialog *sd = new SettingsDialog();
  connect(sd->pushButtonSaveAll, SIGNAL(clicked()), this, SLOT(saveAllSettings()));
  sd->setFont(prevFont);
  sd->setWd(workingDir);
  sd->setTd(texDir);
  sd->setLtd(ltexDir);
  sd->checkBoxSize->setChecked(restoreSize);
  sd->checkBoxPos->setChecked(restorePos);
  if (sd->exec() == QDialog::Accepted) {
    prevFont = sd->getFont();
    workingDir = sd->getWd();
    texDir = sd->getTd();
    ltexDir = sd->getLtd();
    restoreSize = sd->checkBoxSize->isChecked();
    restorePos = sd->checkBoxPos->isChecked();
  }
  delete sd;
}

void TexwizApp::selectBibDb()
{
  QString bibdb = "";
  #ifdef USE_KDE
    bibdb = KFileDialog::getOpenFileName(workingDir,
                                         "BibTeX (*.bib *.BIB|BibTeX Database (*.bib))",
                                         this,
                                         tr("Load BibTeX Database"));
  #else
    bibdb = QFileDialog::getOpenFileName(workingDir,
                                         "BibTeX (*.bib *.BIB)",
                                         this,
                                         tr("Bibliography database"),
                                         tr("Select a BibTeX database"));
  #endif
  if (!bibdb.isEmpty()) lineEditBibDb->setText(bibdb);
}

void TexwizApp::editBibStyles()
{
  QStringList s;
  BibStyleDialog *bd = new BibStyleDialog();

  for (int i = 0; i < comboBoxBibSty->count(); i++) s.append(comboBoxBibSty->text(i));

  bd->listBox->clear();
  bd->listBox->insertStringList(s);
  bd->setDirs(texDir, ltexDir);

  if (bd->exec() == QDialog::Accepted) {
    s.clear();
    for (uint i = 0; i < bd->listBox->count(); i++) s.append(bd->listBox->text(i));
    comboBoxBibSty->clear();
    comboBoxBibSty->insertStringList(s);
    }
  delete bd;
}

void TexwizApp::resizeListBox()
{
  int maxWidth = 10;
  for( QListBoxItem *i = listBox->item(0); i != 0; i = i->next() )
  {
    int w = ((IconListItem *)i)->width(listBox);
    maxWidth = QMAX( w, maxWidth );
  }

  for( QListBoxItem *i = listBox->item(0); i != 0; i = i->next() )
  {
    ((IconListItem *)i)->expandMinimumWidth( maxWidth );
  }

  if( listBox->verticalScrollBar()->isVisible() )
  {
    maxWidth += listBox->verticalScrollBar()->sizeHint().width();
  }

  listBox->setFixedWidth( maxWidth + listBox->frameWidth()*2 );
}

void TexwizApp::whatsThis() {QWhatsThis::enterWhatsThisMode();}

void TexwizApp::saveTemplate()
{
  Template *t = new Template();
  bool ok;
  QString name = QInputDialog::getText(tr("New Template"),
                                       tr("Name of the Template:"),
                                       QLineEdit::Normal,
                                       QString::null, &ok, this );

  if (!ok || name.isEmpty()) {
    delete t;
    return;
  }

  t->setName(name);

  t->setTitle(textEditTitle->text(),
              lineEditAuthor->text(),
              dateEdit->date(),
              checkBoxStaticDate->isChecked());

  t->setDocClass(comboBoxClass->currentItem(),
                 comboBoxPaper->currentItem(),
                 comboBoxFontSize->currentItem(),
                 comboBoxFontFam->currentItem(),
                 comboBoxOutputFormat->currentItem(),
                 comboBoxEncoding->currentItem(),
                 comboBoxLang->currentItem(),
                 checkBoxKoma->isChecked());

  t->setCustomMarkup(checkBoxMarkups->isChecked(), customMarkups);

  t->setTitlePage(buttonGroupTitle->selectedId(), t_head, t_subject,
                  t_publisher, t_thanks, t_extra, t_dedication,
                  tb_top, tb_bottom);

  t->setPageLayout(comboBoxDuplex->currentItem(),
                   comboBoxColumns->currentItem(),
                   comboBoxOrient->currentItem(),
                   comboBoxLSpacing->currentItem(),
                   comboBoxPageStyle->currentItem(),
                   spinBoxFpf->value(),
                   checkBoxSloppy->isChecked());

  t->setLists(checkBoxToc->isChecked(),
              checkBoxLof->isChecked(),
              checkBoxLot->isChecked(),
              checkBoxIndex->isChecked(),
              checkBoxBiblio->isChecked());

  t->setListsToc(checkBoxListsToc->isChecked(),
                 checkBoxIdxToc->isChecked(),
                 checkBoxBibToc->isChecked(),
                 spinBoxTocDepth->value());

  t->setKoma(spinBoxDiv->value(),
             spinBoxBcor->value(),
             comboBoxHeadingSize->currentItem(),
             comboBoxKCaptions->currentItem());

  QStringList lsa, lsb;
  for (int i = 0; i < comboBoxBibSty->count(); i++) lsa.append(comboBoxBibSty->text(i));

  t->setBibtex(buttonGroupBibOpt->selectedId(),
               comboBoxBibSty->currentItem(),
               lineEditBibDb->text(),
               lsa);

  lsa.clear();
  for (int i = 0; i < listBoxPkgAvail->count(); i++) lsa.append(listBoxPkgAvail->text(i));
  for (int i = 0; i < listBoxPkgLoad->count(); i++) lsa.append(listBoxPkgLoad->text(i));

  t->setPackages(lsa, lsb);

  templateList->append(t);
  updateTemplates();
}

void TexwizApp::manageTemplates()
{
  TemplateDialog *tpdlg = new TemplateDialog();
  tpdlg->setTemplates(*templateList);

  if (tpdlg->exec() == QDialog::Accepted) {
    *templateList = tpdlg->getTemplates();
  }

  delete tpdlg;

  updateTemplates();
}

void TexwizApp::setTemplate(int i)
{
  if (i >= templateList->count()) return;
  if (i < 0) return;

  Template *t = templateList->at(i);

  textEditTitle->setText(t->title());
  lineEditAuthor->setText(t->author());
  dateEdit->setDate(t->date());
  checkBoxStaticDate->setChecked(t->staticDate());

  comboBoxClass->setCurrentItem(t->documentClass());
  comboBoxPaper->setCurrentItem(t->paperSize());
  comboBoxFontSize->setCurrentItem(t->fontSize());
  comboBoxFontFam->setCurrentItem(t->fontFam());
  comboBoxOutputFormat->setCurrentItem(t->outputFormat());
  comboBoxEncoding->setCurrentItem(t->encoding());
  comboBoxLang->setCurrentItem(t->language());
  checkBoxKoma->setChecked(t->koma());

  checkBoxMarkups->setChecked(t->useCustomMarkup());

  buttonGroupTitle->setButton(t->titlePageButton());
  t_head = t->titleHead();
  t_subject = t->subject();
  t_publisher = t->publisher();
  t_thanks = t->thanks();
  t_extra = t->extra();
  t_dedication = t->dedication();
  tb_top = t->backsideTop();
  tb_bottom = t->backsideBottom();

  comboBoxDuplex->setCurrentItem(t->duplex());
  comboBoxColumns->setCurrentItem(t->columns());
  comboBoxOrient->setCurrentItem(t->orientation());
  comboBoxLSpacing->setCurrentItem(t->lineSpacing());
  comboBoxPageStyle->setCurrentItem(t->pageStyle());
  spinBoxFpf->setValue(t->floatPageFraction());
  checkBoxSloppy->setChecked(t->sloppy());

  checkBoxToc->setChecked(t->toc());
  checkBoxLof->setChecked(t->lof());
  checkBoxLot->setChecked(t->lot());
  checkBoxIndex->setChecked(t->index());
  checkBoxBiblio->setChecked(t->bibliography());

  checkBoxListsToc->setChecked(t->lofToToc());
  checkBoxIdxToc->setChecked(t->idxToToc());
  checkBoxBibToc->setChecked(t->bibToToc());
  spinBoxTocDepth->setValue(t->tocDepth());

  spinBoxDiv->setValue(t->div());
  spinBoxBcor->setValue(t->bcor());
  comboBoxHeadingSize->setCurrentItem(t->headingSize());
  comboBoxKCaptions->setCurrentItem(t->captions());

  comboBoxBibSty->clear();
  comboBoxBibSty->insertStringList(t->bibtexStylesList());

  buttonGroupBibOpt->setButton(t->bibtexType());
  comboBoxBibSty->setCurrentItem(t->bibtexStyle());
  lineEditBibDb->setText(t->bibtexDatabase());

  listBoxPkgAvail->clear();
  listBoxPkgLoad->clear();
  listBoxPkgAvail->insertStringList(t->packagesAvailable());
  listBoxPkgLoad->insertStringList(t->packagesLoaded());
}

void TexwizApp::updateTemplates()
{
  templateMenu->clear();
  templateMenu->insertItem(tr("Save Template"), this, SLOT(saveTemplate()), 0, -1, -1);
  templateMenu->insertItem(tr("Manage Templates"), this, SLOT(manageTemplates()), 0, -1, -1);

  if (templateList->count() > 0) templateMenu->insertSeparator();

  int i = 0;
  Template *tmpl;
  for (tmpl = templateList->first(); tmpl; tmpl = templateList->next()) {
    templateMenu->insertItem(tmpl->name(), this, SLOT(setTemplate(int)), 0, i, -1);
    templateMenu->setItemParameter(i, i);
    i++;
  }
}
