/***************************************************************************
                          texwiz.h  -  description
                             -------------------
    begin                : Sam Nov 29 16:47:18 CET 2003
    copyright            : (C) 2003 by Nicola D�elin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TEXWIZ_H
#define TEXWIZ_H

#include "texwizmain.h"
#include "template.h"

#ifdef USE_KDE
  #include <kapplication.h>
  #include <dcopclient.h>
#endif

// include files for QT
#include <qevent.h>
#include <qfont.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qsettings.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qvaluevector.h>

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class TexwizView is
  * created which creates your view.
  */
class TexwizApp : public TexwizMain
{
  Q_OBJECT

  public:
    /** construtor */
    TexwizApp();
    /** destructor */
    ~TexwizApp();
    /** initializes all QActions of the application */
    /** shows an about dlg*/

    void setKile(bool b = false);
    void setOutputFile(const QString &of) {outputFile = of;};

  private:
    /** variabeln: */
    bool koma, pdf, twoside, idx, biblio;
    bool listSort, restoreSize, restorePos;
    bool useKile;
    int prev_w, prev_h, prev_x, prev_y;
    QString title, author, date, version;
    QString t_head, t_subject, t_publisher, t_thanks,
            t_dedication, t_extra, tb_top, tb_bottom;
    QString collector, workingDir, texDir, ltexDir;
    QString outputFile;
    QFont prevFont;
    QStringList letterPrivateDetails,
                letterCompanyDetails,
                customMarkups,
                headingOptions,
                oldTemplates;
    QPtrList<Template> *templateList;
    QPopupMenu *templateMenu;

    /** funktionen: */
    QString collectString();
    QString collectLetter();
    QString commentHeader();
    QString collectTitle();
    QString documentClass(bool);
    QString collectLang();
    QString usePackages();
    QString pageStyle();
    QString pageOptions();
    QString pdfOptions();
    QString customMacros();
    QString titlePage();
    void initListBox();
    void initSettings();
    void saveSettings(bool all = false);
    void closeEvent(QCloseEvent *);
    void resizeEvent(QResizeEvent *);
    void showEvent(QShowEvent *);
    void updateTemplates();

  #ifdef USE_KDE
    DCOPClient *client;
  #endif

  private slots:
    void filePreview();
    void fileSaveAs();
    void helpAbout();
    void headingsOptions();
    void slotShowScrhOptions(int);
    void marginChanged();
    void pkgIns();
    void pkgDel();
    void pkgAdd();
    void pkgRem();
    void pkgClear();
    void pkgSort();
    void preferences();
    void saveAllSettings();
    void selectBibDb();
    void editBibStyles();
    void toggleStatusbar(bool);
    void toggleToolbar(bool);
    void toggleKoma(bool);
    void resizeListBox();
    void whatsThis();
    void extendedTitles();
    void editCustomMarkup();
    void updatePreview();
    void saveTemplate();
    void manageTemplates();
    void setTemplate(int);

signals:
    void sigPageStyle(int);
    void sigResize();
};
#endif 

